function [n_harm, n_test, freq, volt, curr, ampspec, blurb] = ...
    ida_read_idf(filename)
%
% Read data in the "D" (Data) file format of the IDA 200 dielectric measurement
% system.
% Note that a line  input_line = strrep(input_line, ',', '.');
% must be uncommented if the input data has commas rather than points as
% the decimal separator.
%
% OUTPUT
%
% n_harm :  the number of harmonics recorded for V and I (usually 8)
% n_test :  the number of test-runs performed
% freq :    n_test x 1 vector of test-frequencies
% volt :    n_test x n_harm matrix of complex voltages for each harmonic
%           and each test
% curr :    n_test x n_harm matrix of complex currents
% ampspec:  array of voltage amplitude values requested in the idf file;
%           these are in peak value form, BUT in some case this array
%           may be empty, e.g. if variables rather than number literals
%           have been used in the idf file
% blurb :   cell array, each element being a line of non-data text from
%           the ida data file;  this may be used together with the
%           ida_write_idf2m function to ensure that this extra information
%           (test object, test time, etc.) is preserved
%
% INPUT
%
% Text string of the filename of the IDA output data (these files are made
% by IDA as "*.idf" )
%

% Nathaniel Taylor, KTH.
% 2003-11-xx: first version.
% 2004-02-10 & 2004-02-24:  cross-check impedances.
% 2005-04-04: get specified voltages (Amplitude  =  ...)
% 2009-05-xx: adopt for use with new IDAX300 (just comments).
%


% Set the text that comes before the numerical data for f-Z, V and I:
point_data_string = 'Point(';
volt_data_string = 'Channel(0 ''Voltage''' ;
curr_data_string = 'Channel(1 ''Current''' ;

%% File input and output initialisation. %%

% check input
if isempty(filename),
    error('Input filename was not given (was empty)')
end
% open input file
infile = fopen(filename,'r');
if infile == -1,
    error([' Couldn''t open the input file ', filename, ' for reading.'])
end


%% Read data in. %%
% take a line at a time, interpret it if it matches a desired data line,
% else store it in memory as a string

% the lengths of the demarcation strings
l_point_str = length(point_data_string);
l_volt_str = length(volt_data_string);
l_curr_str = length(curr_data_string);

% initialise variables to hold data
blurb = {};  % for all header lines
n_point=0; n_volt=0; n_curr=0; n_blurb=0;
freq = [];  points = [];  impedances = [];  volt = [];  curr = [];

% default for the rms/peak choice in the icf file
amplitude_scale = 1;
% initialise this first, just in case there isn't an "Amplitude" line
% (as the Amplitude bit is an "extra", it should never cause failure)
ampspec = [];

data_found = 0;
while 1,
    input_line = fgets(infile);
    %input_line = strrep(input_line, ',', '.');
    l_line = length(input_line);

    if input_line == -1,  % end of file
        if data_found == 0,
            error('End Of File reached without finding data')
        else
            break
        end
    end

    if ( l_line > l_point_str && strcmp(input_line(1:l_point_str), point_data_string) ),
        % a 'points' line;  the first numerical value is a the measurement-frequency
        % further points are complex impedances, i.e. v/i
        data_found = 1;
        n_point = n_point+1;
        temp = strrep(input_line, point_data_string, '');
        temp = strrep(temp, ')', '');
        temp = str2num(temp);
        freq = [ freq; temp(1,1) ];
        if length(temp)>1,
            impedances = [ impedances ; temp(1,2:length(temp)) ];
        end

    elseif ( l_line > l_volt_str  &&  strcmp(input_line(1:l_volt_str),volt_data_string) ),
        % a line specifying voltages
        n_volt = n_volt+1;
        temp = strrep(input_line, volt_data_string, '');
        temp = strrep(temp, ')', '');
        temp = str2num(temp);
        volt = [ volt; temp ];

    elseif ( l_line > l_curr_str && strcmp(input_line(1:l_curr_str),curr_data_string) ),
        % a line specifying currents
        n_curr = n_curr+1;
        temp = strrep(input_line, curr_data_string, '');
        temp = strrep(temp, ')', '');
        temp = str2num(temp);
        curr = [ curr; temp ];

    else
        % store other lines in memory, to write as comments later
        n_blurb = n_blurb+1;
        blurb{n_blurb,1} = input_line;

        % find if Peak or RMS is meant;  the apparently superfluous
        % default value _and_ check for 'Peak' is in case contradictory
        % (-> latest assertion should be used) or no (-> use the default)
        % AmplitudeScale information is given.
        if regexpi(input_line, '^\s*AmplitudeScale\s*=\s*''Peak''')
            amplitude_scale = 1;
        elseif regexpi(input_line, '^\s*AmplitudeScale\s*=\s*''RMS''')
            amplitude_scale = 1/sqrt(2);
        end

        % a line specifying amplitudes to use: if variables are used within
        % this then the methods below fail;  else, it is hoped the correct
        % result will be returned...
        amplitude_re = '^\s*Amplitude\s*=' ;
        if regexpi(input_line, amplitude_re)
            % remove all before and after the desired numbers
            ampspec = regexprep(input_line, amplitude_re, '', 'ignorecase');
            ampspec = regexprep(ampspec, '%.*$', '', 'ignorecase');
            % identify whether the input was plain numbers or a sweep or unknown:
            if regexpi(ampspec,'LinSweep')
                amplitude_spec_type = 2;  % a linsweep
                ampspec = regexprep(ampspec, 'LinSweep', '', 'ignorecase');
            elseif regexpi(ampspec,'LogSweep')
                amplitude_spec_type = 3;
                ampspec = regexprep(ampspec, 'LogSweep', '', 'ignorecase');
            elseif ~isempty(regexpi(ampspec,'[a-z]')) | isempty(regexpi(ampspec,'\d'))
                amplitude_spec_type = -1;
            else
                amplitude_spec_type = 1;
            end
            % then get rid of all non-numeric characters
            ampspec = regexprep( ampspec, '\D+', ' ');
            % and convert to a numeric array
            ampspec = eval([ '[', ampspec, ']' ]);
            % now, if a special meaning (Sweep) exists for the numbers, use
            % this properly, else output the number directly or output
            % an empty array if the format was not properly recognised
            if amplitude_spec_type==2 && length(ampspec)==3
                ampspec = ampspec(1):ampspec(3):ampspec(2) / amplitude_scale;
            elseif amplitude_spec_type==3 && length(ampspec)==3
                tempfrom = log10(ampspec(1));
                tempto = log10(ampspec(2));
                decades = max([tempfrom,tempto]) - min([tempfrom,tempto]);
                ampspec = logspace(tempfrom, tempto, ampspec(3)*decades+1) ...
                    / amplitude_scale;
            elseif amplitude_spec_type==1
                ampspec = ampspec / amplitude_scale;
            else
                ampspec = [];
            end
        end

        %% END of added section

    end
end

% find matrix sizes, check basic consistency
[n_test, n_harm] = size(volt);
if ( sum(size(volt)~=size(curr))  ||  ( ~isempty(impedances) && sum(size(volt)~=size(impedances)) ) ),
    [volt_rows, volt_columns] = size(volt)
    [curr_rows, curr_columns] = size(curr)
    if ~isempty(impedances),
        [z_rows, z_columns] = size(impedances)
    end
    error('The voltage, current and impedance matrices extracted are not all of equal size')
end

% check that v and i values match with z values
if ~isempty(impedances),
    err_tol = 1e-5;
    errs = (abs(volt./curr)-abs(impedances))./sqrt(abs(volt./curr).*abs(impedances));
    if ~isempty( find( abs(errs)>err_tol ) ),
        errs
    end

else
    disp(['*** ida_read_idf: no impedance data was found, so there', ...
        ' is no verification of v,i matching f points correctly'])
end

